# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _mesh_operators
else:
    import _mesh_operators

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _mesh_operators.SWIG_PyInstanceMethod_New
_swig_new_static_method = _mesh_operators.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._ser.array
import mfem._ser.mem_manager
import mfem._ser.vector
import mfem._ser.mesh
import mfem._ser.matrix
import mfem._ser.operators
import mfem._ser.sort_pairs
import mfem._ser.ncmesh
import mfem._ser.gridfunc
import mfem._ser.coefficient
import mfem._ser.globals
import mfem._ser.intrules
import mfem._ser.sparsemat
import mfem._ser.densemat
import mfem._ser.eltrans
import mfem._ser.fe
import mfem._ser.geom
import mfem._ser.fespace
import mfem._ser.fe_coll
import mfem._ser.lininteg
import mfem._ser.handle
import mfem._ser.restriction
import mfem._ser.element
import mfem._ser.table
import mfem._ser.hash
import mfem._ser.bilininteg
import mfem._ser.linearform
import mfem._ser.vertex
import mfem._ser.vtk
import mfem._ser.estimators
import mfem._ser.bilinearform
class MeshOperator(object):
    r"""Proxy of C++ mfem::MeshOperator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    NONE = _mesh_operators.MeshOperator_NONE
    
    CONTINUE = _mesh_operators.MeshOperator_CONTINUE
    
    STOP = _mesh_operators.MeshOperator_STOP
    
    REPEAT = _mesh_operators.MeshOperator_REPEAT
    
    MASK_UPDATE = _mesh_operators.MeshOperator_MASK_UPDATE
    
    MASK_ACTION = _mesh_operators.MeshOperator_MASK_ACTION
    
    REFINED = _mesh_operators.MeshOperator_REFINED
    
    DEREFINED = _mesh_operators.MeshOperator_DEREFINED
    
    REBALANCED = _mesh_operators.MeshOperator_REBALANCED
    
    MASK_INFO = _mesh_operators.MeshOperator_MASK_INFO
    

    def Apply(self, mesh):
        r"""Apply(MeshOperator self, Mesh mesh) -> bool"""
        return _mesh_operators.MeshOperator_Apply(self, mesh)
    Apply = _swig_new_instance_method(_mesh_operators.MeshOperator_Apply)

    def Stop(self):
        r"""Stop(MeshOperator self) -> bool"""
        return _mesh_operators.MeshOperator_Stop(self)
    Stop = _swig_new_instance_method(_mesh_operators.MeshOperator_Stop)

    def Repeat(self):
        r"""Repeat(MeshOperator self) -> bool"""
        return _mesh_operators.MeshOperator_Repeat(self)
    Repeat = _swig_new_instance_method(_mesh_operators.MeshOperator_Repeat)

    def Continue(self):
        r"""Continue(MeshOperator self) -> bool"""
        return _mesh_operators.MeshOperator_Continue(self)
    Continue = _swig_new_instance_method(_mesh_operators.MeshOperator_Continue)

    def Refined(self):
        r"""Refined(MeshOperator self) -> bool"""
        return _mesh_operators.MeshOperator_Refined(self)
    Refined = _swig_new_instance_method(_mesh_operators.MeshOperator_Refined)

    def Derefined(self):
        r"""Derefined(MeshOperator self) -> bool"""
        return _mesh_operators.MeshOperator_Derefined(self)
    Derefined = _swig_new_instance_method(_mesh_operators.MeshOperator_Derefined)

    def Rebalanced(self):
        r"""Rebalanced(MeshOperator self) -> bool"""
        return _mesh_operators.MeshOperator_Rebalanced(self)
    Rebalanced = _swig_new_instance_method(_mesh_operators.MeshOperator_Rebalanced)

    def GetActionInfo(self):
        r"""GetActionInfo(MeshOperator self) -> int"""
        return _mesh_operators.MeshOperator_GetActionInfo(self)
    GetActionInfo = _swig_new_instance_method(_mesh_operators.MeshOperator_GetActionInfo)

    def Reset(self):
        r"""Reset(MeshOperator self)"""
        return _mesh_operators.MeshOperator_Reset(self)
    Reset = _swig_new_instance_method(_mesh_operators.MeshOperator_Reset)
    __swig_destroy__ = _mesh_operators.delete_MeshOperator

# Register MeshOperator in _mesh_operators:
_mesh_operators.MeshOperator_swigregister(MeshOperator)

class MeshOperatorSequence(MeshOperator):
    r"""Proxy of C++ mfem::MeshOperatorSequence class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(MeshOperatorSequence self) -> MeshOperatorSequence"""
        _mesh_operators.MeshOperatorSequence_swiginit(self, _mesh_operators.new_MeshOperatorSequence())
    __swig_destroy__ = _mesh_operators.delete_MeshOperatorSequence

    def Append(self, mc):
        r"""Append(MeshOperatorSequence self, MeshOperator mc)"""
        return _mesh_operators.MeshOperatorSequence_Append(self, mc)
    Append = _swig_new_instance_method(_mesh_operators.MeshOperatorSequence_Append)

    def GetSequence(self):
        r"""GetSequence(MeshOperatorSequence self) -> mfem::Array< mfem::MeshOperator * > &"""
        return _mesh_operators.MeshOperatorSequence_GetSequence(self)
    GetSequence = _swig_new_instance_method(_mesh_operators.MeshOperatorSequence_GetSequence)

    def Reset(self):
        r"""Reset(MeshOperatorSequence self)"""
        return _mesh_operators.MeshOperatorSequence_Reset(self)
    Reset = _swig_new_instance_method(_mesh_operators.MeshOperatorSequence_Reset)

# Register MeshOperatorSequence in _mesh_operators:
_mesh_operators.MeshOperatorSequence_swigregister(MeshOperatorSequence)

class ThresholdRefiner(MeshOperator):
    r"""Proxy of C++ mfem::ThresholdRefiner class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, est):
        r"""__init__(ThresholdRefiner self, ErrorEstimator est) -> ThresholdRefiner"""
        _mesh_operators.ThresholdRefiner_swiginit(self, _mesh_operators.new_ThresholdRefiner(est))

    def SetTotalErrorNormP(self, *args, **kwargs):
        r"""SetTotalErrorNormP(ThresholdRefiner self, double norm_p=mfem::infinity())"""
        return _mesh_operators.ThresholdRefiner_SetTotalErrorNormP(self, *args, **kwargs)
    SetTotalErrorNormP = _swig_new_instance_method(_mesh_operators.ThresholdRefiner_SetTotalErrorNormP)

    def SetTotalErrorGoal(self, err_goal):
        r"""SetTotalErrorGoal(ThresholdRefiner self, double err_goal)"""
        return _mesh_operators.ThresholdRefiner_SetTotalErrorGoal(self, err_goal)
    SetTotalErrorGoal = _swig_new_instance_method(_mesh_operators.ThresholdRefiner_SetTotalErrorGoal)

    def SetTotalErrorFraction(self, fraction):
        r"""SetTotalErrorFraction(ThresholdRefiner self, double fraction)"""
        return _mesh_operators.ThresholdRefiner_SetTotalErrorFraction(self, fraction)
    SetTotalErrorFraction = _swig_new_instance_method(_mesh_operators.ThresholdRefiner_SetTotalErrorFraction)

    def SetLocalErrorGoal(self, err_goal):
        r"""SetLocalErrorGoal(ThresholdRefiner self, double err_goal)"""
        return _mesh_operators.ThresholdRefiner_SetLocalErrorGoal(self, err_goal)
    SetLocalErrorGoal = _swig_new_instance_method(_mesh_operators.ThresholdRefiner_SetLocalErrorGoal)

    def SetMaxElements(self, max_elem):
        r"""SetMaxElements(ThresholdRefiner self, long max_elem)"""
        return _mesh_operators.ThresholdRefiner_SetMaxElements(self, max_elem)
    SetMaxElements = _swig_new_instance_method(_mesh_operators.ThresholdRefiner_SetMaxElements)

    def PreferNonconformingRefinement(self):
        r"""PreferNonconformingRefinement(ThresholdRefiner self)"""
        return _mesh_operators.ThresholdRefiner_PreferNonconformingRefinement(self)
    PreferNonconformingRefinement = _swig_new_instance_method(_mesh_operators.ThresholdRefiner_PreferNonconformingRefinement)

    def PreferConformingRefinement(self):
        r"""PreferConformingRefinement(ThresholdRefiner self)"""
        return _mesh_operators.ThresholdRefiner_PreferConformingRefinement(self)
    PreferConformingRefinement = _swig_new_instance_method(_mesh_operators.ThresholdRefiner_PreferConformingRefinement)

    def SetNCLimit(self, nc_limit):
        r"""SetNCLimit(ThresholdRefiner self, int nc_limit)"""
        return _mesh_operators.ThresholdRefiner_SetNCLimit(self, nc_limit)
    SetNCLimit = _swig_new_instance_method(_mesh_operators.ThresholdRefiner_SetNCLimit)

    def GetNumMarkedElements(self):
        r"""GetNumMarkedElements(ThresholdRefiner self) -> long"""
        return _mesh_operators.ThresholdRefiner_GetNumMarkedElements(self)
    GetNumMarkedElements = _swig_new_instance_method(_mesh_operators.ThresholdRefiner_GetNumMarkedElements)

    def GetThreshold(self):
        r"""GetThreshold(ThresholdRefiner self) -> double"""
        return _mesh_operators.ThresholdRefiner_GetThreshold(self)
    GetThreshold = _swig_new_instance_method(_mesh_operators.ThresholdRefiner_GetThreshold)

    def Reset(self):
        r"""Reset(ThresholdRefiner self)"""
        return _mesh_operators.ThresholdRefiner_Reset(self)
    Reset = _swig_new_instance_method(_mesh_operators.ThresholdRefiner_Reset)
    __swig_destroy__ = _mesh_operators.delete_ThresholdRefiner

# Register ThresholdRefiner in _mesh_operators:
_mesh_operators.ThresholdRefiner_swigregister(ThresholdRefiner)

class ThresholdDerefiner(MeshOperator):
    r"""Proxy of C++ mfem::ThresholdDerefiner class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, est):
        r"""__init__(ThresholdDerefiner self, ErrorEstimator est) -> ThresholdDerefiner"""
        _mesh_operators.ThresholdDerefiner_swiginit(self, _mesh_operators.new_ThresholdDerefiner(est))

    def SetThreshold(self, thresh):
        r"""SetThreshold(ThresholdDerefiner self, double thresh)"""
        return _mesh_operators.ThresholdDerefiner_SetThreshold(self, thresh)
    SetThreshold = _swig_new_instance_method(_mesh_operators.ThresholdDerefiner_SetThreshold)

    def SetOp(self, op):
        r"""SetOp(ThresholdDerefiner self, int op)"""
        return _mesh_operators.ThresholdDerefiner_SetOp(self, op)
    SetOp = _swig_new_instance_method(_mesh_operators.ThresholdDerefiner_SetOp)

    def SetNCLimit(self, nc_limit):
        r"""SetNCLimit(ThresholdDerefiner self, int nc_limit)"""
        return _mesh_operators.ThresholdDerefiner_SetNCLimit(self, nc_limit)
    SetNCLimit = _swig_new_instance_method(_mesh_operators.ThresholdDerefiner_SetNCLimit)

    def Reset(self):
        r"""Reset(ThresholdDerefiner self)"""
        return _mesh_operators.ThresholdDerefiner_Reset(self)
    Reset = _swig_new_instance_method(_mesh_operators.ThresholdDerefiner_Reset)
    __swig_destroy__ = _mesh_operators.delete_ThresholdDerefiner

# Register ThresholdDerefiner in _mesh_operators:
_mesh_operators.ThresholdDerefiner_swigregister(ThresholdDerefiner)

class Rebalancer(MeshOperator):
    r"""Proxy of C++ mfem::Rebalancer class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def Reset(self):
        r"""Reset(Rebalancer self)"""
        return _mesh_operators.Rebalancer_Reset(self)
    Reset = _swig_new_instance_method(_mesh_operators.Rebalancer_Reset)

    def __init__(self):
        r"""__init__(Rebalancer self) -> Rebalancer"""
        _mesh_operators.Rebalancer_swiginit(self, _mesh_operators.new_Rebalancer())
    __swig_destroy__ = _mesh_operators.delete_Rebalancer

# Register Rebalancer in _mesh_operators:
_mesh_operators.Rebalancer_swigregister(Rebalancer)






from __future__ import print_function
import os
import sys
import numpy as np

if len(sys.argv) > 1 and sys.argv[1] == '-p':   
    import mfem.par as mfem
    use_parallel = True
    from mfem.common.mpi_debug import nicePrint as print
    from mpi4py import MPI
    myid  = MPI.COMM_WORLD.rank
    
else:
    import mfem.ser as mfem
    use_parallel = False
    myid = 0
    
def run_test(): 
    a = mfem.intArray([1,2,3])

    for i in range(a.Size()):
        print(a[i])
    a.Print()
    a.Print("intArray.dat")
    a.Save("intArray_save.dat")
    
    b = mfem.doubleArray([1.1,2.2,3.])
    for i in range(b.Size()):
        print(b[i])
    b.Print()
    b.Print("doubleArray.dat")
    b.Save("doubleArray_save.dat")    
    
if __name__=='__main__':
    run_test()
 



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _complex_operator
else:
    import _complex_operator

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _complex_operator.SWIG_PyInstanceMethod_New
_swig_new_static_method = _complex_operator.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._par.vector
import mfem._par.array
import mfem._par.mem_manager
import mfem._par.operators
import mfem._par.sparsemat
import mfem._par.matrix
import mfem._par.densemat
class ComplexOperator(mfem._par.operators.Operator):
    r"""Proxy of C++ mfem::ComplexOperator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    HERMITIAN = _complex_operator.ComplexOperator_HERMITIAN
    
    BLOCK_SYMMETRIC = _complex_operator.ComplexOperator_BLOCK_SYMMETRIC
    
    __swig_destroy__ = _complex_operator.delete_ComplexOperator

    def hasRealPart(self):
        r"""hasRealPart(ComplexOperator self) -> bool"""
        return _complex_operator.ComplexOperator_hasRealPart(self)
    hasRealPart = _swig_new_instance_method(_complex_operator.ComplexOperator_hasRealPart)

    def hasImagPart(self):
        r"""hasImagPart(ComplexOperator self) -> bool"""
        return _complex_operator.ComplexOperator_hasImagPart(self)
    hasImagPart = _swig_new_instance_method(_complex_operator.ComplexOperator_hasImagPart)

    def real(self, *args):
        r"""
        real(ComplexOperator self) -> Operator
        real(ComplexOperator self) -> Operator
        """
        return _complex_operator.ComplexOperator_real(self, *args)
    real = _swig_new_instance_method(_complex_operator.ComplexOperator_real)

    def imag(self, *args):
        r"""
        imag(ComplexOperator self) -> Operator
        imag(ComplexOperator self) -> Operator
        """
        return _complex_operator.ComplexOperator_imag(self, *args)
    imag = _swig_new_instance_method(_complex_operator.ComplexOperator_imag)

    def Mult(self, x, y):
        r"""Mult(ComplexOperator self, Vector x, Vector y)"""
        return _complex_operator.ComplexOperator_Mult(self, x, y)
    Mult = _swig_new_instance_method(_complex_operator.ComplexOperator_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(ComplexOperator self, Vector x, Vector y)"""
        return _complex_operator.ComplexOperator_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_complex_operator.ComplexOperator_MultTranspose)

    def GetType(self):
        r"""GetType(ComplexOperator self) -> mfem::Operator::Type"""
        return _complex_operator.ComplexOperator_GetType(self)
    GetType = _swig_new_instance_method(_complex_operator.ComplexOperator_GetType)

    def GetConvention(self):
        r"""GetConvention(ComplexOperator self) -> mfem::ComplexOperator::Convention"""
        return _complex_operator.ComplexOperator_GetConvention(self)
    GetConvention = _swig_new_instance_method(_complex_operator.ComplexOperator_GetConvention)

    def __init__(self, Op_Real, Op_Imag, ownReal=False, ownImag=False, hermitan=True):
        r"""__init__(ComplexOperator self, Operator Op_Real, Operator Op_Imag, bool ownReal=False, bool ownImag=False, bool hermitan=True) -> ComplexOperator"""

        self._parts = [Op_Real, Op_Imag]
        if ownReal:
           assert Op_Real.thisown != 0, "Real Op is not owned by passed object"
           Op_Real.thisown = 0
        if ownImag:
           assert Op_Real.thisown != 0, "Imag Op is not owned by passed object"     
           Op_Imag.thisown = 0


        _complex_operator.ComplexOperator_swiginit(self, _complex_operator.new_ComplexOperator(Op_Real, Op_Imag, ownReal, ownImag, hermitan))

# Register ComplexOperator in _complex_operator:
_complex_operator.ComplexOperator_swigregister(ComplexOperator)

class ComplexSparseMatrix(ComplexOperator):
    r"""Proxy of C++ mfem::ComplexSparseMatrix class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args, **kwargs):
        r"""__init__(ComplexSparseMatrix self, SparseMatrix A_Real, SparseMatrix A_Imag, bool ownReal, bool ownImag, mfem::ComplexOperator::Convention convention=HERMITIAN) -> ComplexSparseMatrix"""
        _complex_operator.ComplexSparseMatrix_swiginit(self, _complex_operator.new_ComplexSparseMatrix(*args, **kwargs))

    def real(self, *args):
        r"""
        real(ComplexSparseMatrix self) -> SparseMatrix
        real(ComplexSparseMatrix self) -> SparseMatrix
        """
        return _complex_operator.ComplexSparseMatrix_real(self, *args)
    real = _swig_new_instance_method(_complex_operator.ComplexSparseMatrix_real)

    def imag(self, *args):
        r"""
        imag(ComplexSparseMatrix self) -> SparseMatrix
        imag(ComplexSparseMatrix self) -> SparseMatrix
        """
        return _complex_operator.ComplexSparseMatrix_imag(self, *args)
    imag = _swig_new_instance_method(_complex_operator.ComplexSparseMatrix_imag)

    def GetSystemMatrix(self):
        r"""GetSystemMatrix(ComplexSparseMatrix self) -> SparseMatrix"""
        return _complex_operator.ComplexSparseMatrix_GetSystemMatrix(self)
    GetSystemMatrix = _swig_new_instance_method(_complex_operator.ComplexSparseMatrix_GetSystemMatrix)

    def GetType(self):
        r"""GetType(ComplexSparseMatrix self) -> mfem::Operator::Type"""
        return _complex_operator.ComplexSparseMatrix_GetType(self)
    GetType = _swig_new_instance_method(_complex_operator.ComplexSparseMatrix_GetType)
    __swig_destroy__ = _complex_operator.delete_ComplexSparseMatrix

# Register ComplexSparseMatrix in _complex_operator:
_complex_operator.ComplexSparseMatrix_swigregister(ComplexSparseMatrix)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _hybridization
else:
    import _hybridization

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _hybridization.SWIG_PyInstanceMethod_New
_swig_new_static_method = _hybridization.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._par.handle
import mfem._par.operators
import mfem._par.mem_manager
import mfem._par.vector
import mfem._par.array
import mfem._par.hypre
import mfem._par.globals
import mfem._par.sparsemat
import mfem._par.matrix
import mfem._par.densemat
import mfem._par.fespace
import mfem._par.coefficient
import mfem._par.intrules
import mfem._par.eltrans
import mfem._par.fe
import mfem._par.geom
import mfem._par.mesh
import mfem._par.sort_pairs
import mfem._par.ncmesh
import mfem._par.vtk
import mfem._par.element
import mfem._par.table
import mfem._par.hash
import mfem._par.vertex
import mfem._par.gridfunc
import mfem._par.bilininteg
import mfem._par.fe_coll
import mfem._par.lininteg
import mfem._par.linearform
import mfem._par.restriction
class Hybridization(object):
    r"""Proxy of C++ mfem::Hybridization class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, fespace, c_fespace):
        r"""__init__(Hybridization self, FiniteElementSpace fespace, FiniteElementSpace c_fespace) -> Hybridization"""
        _hybridization.Hybridization_swiginit(self, _hybridization.new_Hybridization(fespace, c_fespace))
    __swig_destroy__ = _hybridization.delete_Hybridization

    def SetConstraintIntegrator(self, c_integ):
        r"""SetConstraintIntegrator(Hybridization self, BilinearFormIntegrator c_integ)"""
        return _hybridization.Hybridization_SetConstraintIntegrator(self, c_integ)
    SetConstraintIntegrator = _swig_new_instance_method(_hybridization.Hybridization_SetConstraintIntegrator)

    def Init(self, ess_tdof_list):
        r"""Init(Hybridization self, intArray ess_tdof_list)"""
        return _hybridization.Hybridization_Init(self, ess_tdof_list)
    Init = _swig_new_instance_method(_hybridization.Hybridization_Init)

    def AssembleMatrix(self, el, A):
        r"""AssembleMatrix(Hybridization self, int el, DenseMatrix A)"""
        return _hybridization.Hybridization_AssembleMatrix(self, el, A)
    AssembleMatrix = _swig_new_instance_method(_hybridization.Hybridization_AssembleMatrix)

    def AssembleBdrMatrix(self, bdr_el, A):
        r"""AssembleBdrMatrix(Hybridization self, int bdr_el, DenseMatrix A)"""
        return _hybridization.Hybridization_AssembleBdrMatrix(self, bdr_el, A)
    AssembleBdrMatrix = _swig_new_instance_method(_hybridization.Hybridization_AssembleBdrMatrix)

    def Finalize(self):
        r"""Finalize(Hybridization self)"""
        return _hybridization.Hybridization_Finalize(self)
    Finalize = _swig_new_instance_method(_hybridization.Hybridization_Finalize)

    def GetMatrix(self):
        r"""GetMatrix(Hybridization self) -> SparseMatrix"""
        return _hybridization.Hybridization_GetMatrix(self)
    GetMatrix = _swig_new_instance_method(_hybridization.Hybridization_GetMatrix)

    def GetParallelMatrix(self, *args):
        r"""
        GetParallelMatrix(Hybridization self) -> HypreParMatrix
        GetParallelMatrix(Hybridization self, OperatorHandle H_h)
        """
        return _hybridization.Hybridization_GetParallelMatrix(self, *args)
    GetParallelMatrix = _swig_new_instance_method(_hybridization.Hybridization_GetParallelMatrix)

    def SetOperatorType(self, tid):
        r"""SetOperatorType(Hybridization self, mfem::Operator::Type tid)"""
        return _hybridization.Hybridization_SetOperatorType(self, tid)
    SetOperatorType = _swig_new_instance_method(_hybridization.Hybridization_SetOperatorType)

    def ReduceRHS(self, b, b_r):
        r"""ReduceRHS(Hybridization self, Vector b, Vector b_r)"""
        return _hybridization.Hybridization_ReduceRHS(self, b, b_r)
    ReduceRHS = _swig_new_instance_method(_hybridization.Hybridization_ReduceRHS)

    def ComputeSolution(self, b, sol_r, sol):
        r"""ComputeSolution(Hybridization self, Vector b, Vector sol_r, Vector sol)"""
        return _hybridization.Hybridization_ComputeSolution(self, b, sol_r, sol)
    ComputeSolution = _swig_new_instance_method(_hybridization.Hybridization_ComputeSolution)

    def Reset(self):
        r"""Reset(Hybridization self)"""
        return _hybridization.Hybridization_Reset(self)
    Reset = _swig_new_instance_method(_hybridization.Hybridization_Reset)

# Register Hybridization in _hybridization:
_hybridization.Hybridization_swigregister(Hybridization)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _datacollection
else:
    import _datacollection

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _datacollection.SWIG_PyInstanceMethod_New
_swig_new_static_method = _datacollection.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

import mfem._ser.globals
import mfem._ser.mesh
import mfem._ser.matrix
import mfem._ser.vector
import mfem._ser.array
import mfem._ser.mem_manager
import mfem._ser.operators
import mfem._ser.sort_pairs
import mfem._ser.ncmesh
import mfem._ser.gridfunc
import mfem._ser.coefficient
import mfem._ser.intrules
import mfem._ser.sparsemat
import mfem._ser.densemat
import mfem._ser.eltrans
import mfem._ser.fe
import mfem._ser.geom
import mfem._ser.fespace
import mfem._ser.fe_coll
import mfem._ser.lininteg
import mfem._ser.handle
import mfem._ser.restriction
import mfem._ser.element
import mfem._ser.table
import mfem._ser.hash
import mfem._ser.bilininteg
import mfem._ser.linearform
import mfem._ser.vertex
import mfem._ser.vtk
class DataCollection(object):
    r"""Proxy of C++ mfem::DataCollection class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    SERIAL_FORMAT = _datacollection.DataCollection_SERIAL_FORMAT
    
    PARALLEL_FORMAT = _datacollection.DataCollection_PARALLEL_FORMAT
    

    def __init__(self, collection_name, mesh_=None):
        r"""__init__(DataCollection self, std::string const & collection_name, Mesh mesh_=None) -> DataCollection"""
        _datacollection.DataCollection_swiginit(self, _datacollection.new_DataCollection(collection_name, mesh_))

    def RegisterField(self, field_name, gf):
        r"""RegisterField(DataCollection self, std::string const & field_name, GridFunction gf)"""
        return _datacollection.DataCollection_RegisterField(self, field_name, gf)
    RegisterField = _swig_new_instance_method(_datacollection.DataCollection_RegisterField)

    def DeregisterField(self, field_name):
        r"""DeregisterField(DataCollection self, std::string const & field_name)"""
        return _datacollection.DataCollection_DeregisterField(self, field_name)
    DeregisterField = _swig_new_instance_method(_datacollection.DataCollection_DeregisterField)

    def RegisterQField(self, q_field_name, qf):
        r"""RegisterQField(DataCollection self, std::string const & q_field_name, QuadratureFunction qf)"""
        return _datacollection.DataCollection_RegisterQField(self, q_field_name, qf)
    RegisterQField = _swig_new_instance_method(_datacollection.DataCollection_RegisterQField)

    def DeregisterQField(self, field_name):
        r"""DeregisterQField(DataCollection self, std::string const & field_name)"""
        return _datacollection.DataCollection_DeregisterQField(self, field_name)
    DeregisterQField = _swig_new_instance_method(_datacollection.DataCollection_DeregisterQField)

    def HasField(self, name):
        r"""HasField(DataCollection self, std::string const & name) -> bool"""
        return _datacollection.DataCollection_HasField(self, name)
    HasField = _swig_new_instance_method(_datacollection.DataCollection_HasField)

    def GetField(self, field_name):
        r"""GetField(DataCollection self, std::string const & field_name) -> GridFunction"""
        return _datacollection.DataCollection_GetField(self, field_name)
    GetField = _swig_new_instance_method(_datacollection.DataCollection_GetField)

    def HasQField(self, q_field_name):
        r"""HasQField(DataCollection self, std::string const & q_field_name) -> bool"""
        return _datacollection.DataCollection_HasQField(self, q_field_name)
    HasQField = _swig_new_instance_method(_datacollection.DataCollection_HasQField)

    def GetQField(self, q_field_name):
        r"""GetQField(DataCollection self, std::string const & q_field_name) -> QuadratureFunction"""
        return _datacollection.DataCollection_GetQField(self, q_field_name)
    GetQField = _swig_new_instance_method(_datacollection.DataCollection_GetQField)

    def GetFieldMap(self):
        r"""GetFieldMap(DataCollection self) -> mfem::DataCollection::FieldMapType const &"""
        return _datacollection.DataCollection_GetFieldMap(self)
    GetFieldMap = _swig_new_instance_method(_datacollection.DataCollection_GetFieldMap)

    def GetQFieldMap(self):
        r"""GetQFieldMap(DataCollection self) -> mfem::DataCollection::QFieldMapType const &"""
        return _datacollection.DataCollection_GetQFieldMap(self)
    GetQFieldMap = _swig_new_instance_method(_datacollection.DataCollection_GetQFieldMap)

    def GetMesh(self):
        r"""GetMesh(DataCollection self) -> Mesh"""
        return _datacollection.DataCollection_GetMesh(self)
    GetMesh = _swig_new_instance_method(_datacollection.DataCollection_GetMesh)

    def SetMesh(self, new_mesh):
        r"""SetMesh(DataCollection self, Mesh new_mesh)"""
        return _datacollection.DataCollection_SetMesh(self, new_mesh)
    SetMesh = _swig_new_instance_method(_datacollection.DataCollection_SetMesh)

    def SetCycle(self, c):
        r"""SetCycle(DataCollection self, int c)"""
        return _datacollection.DataCollection_SetCycle(self, c)
    SetCycle = _swig_new_instance_method(_datacollection.DataCollection_SetCycle)

    def SetTime(self, t):
        r"""SetTime(DataCollection self, double t)"""
        return _datacollection.DataCollection_SetTime(self, t)
    SetTime = _swig_new_instance_method(_datacollection.DataCollection_SetTime)

    def SetTimeStep(self, ts):
        r"""SetTimeStep(DataCollection self, double ts)"""
        return _datacollection.DataCollection_SetTimeStep(self, ts)
    SetTimeStep = _swig_new_instance_method(_datacollection.DataCollection_SetTimeStep)

    def GetCycle(self):
        r"""GetCycle(DataCollection self) -> int"""
        return _datacollection.DataCollection_GetCycle(self)
    GetCycle = _swig_new_instance_method(_datacollection.DataCollection_GetCycle)

    def GetTime(self):
        r"""GetTime(DataCollection self) -> double"""
        return _datacollection.DataCollection_GetTime(self)
    GetTime = _swig_new_instance_method(_datacollection.DataCollection_GetTime)

    def GetTimeStep(self):
        r"""GetTimeStep(DataCollection self) -> double"""
        return _datacollection.DataCollection_GetTimeStep(self)
    GetTimeStep = _swig_new_instance_method(_datacollection.DataCollection_GetTimeStep)

    def GetCollectionName(self):
        r"""GetCollectionName(DataCollection self) -> std::string const &"""
        return _datacollection.DataCollection_GetCollectionName(self)
    GetCollectionName = _swig_new_instance_method(_datacollection.DataCollection_GetCollectionName)

    def SetOwnData(self, o):
        r"""SetOwnData(DataCollection self, bool o)"""
        return _datacollection.DataCollection_SetOwnData(self, o)
    SetOwnData = _swig_new_instance_method(_datacollection.DataCollection_SetOwnData)

    def SetPrecision(self, prec):
        r"""SetPrecision(DataCollection self, int prec)"""
        return _datacollection.DataCollection_SetPrecision(self, prec)
    SetPrecision = _swig_new_instance_method(_datacollection.DataCollection_SetPrecision)

    def SetPadDigits(self, digits):
        r"""SetPadDigits(DataCollection self, int digits)"""
        return _datacollection.DataCollection_SetPadDigits(self, digits)
    SetPadDigits = _swig_new_instance_method(_datacollection.DataCollection_SetPadDigits)

    def SetPadDigitsCycle(self, digits):
        r"""SetPadDigitsCycle(DataCollection self, int digits)"""
        return _datacollection.DataCollection_SetPadDigitsCycle(self, digits)
    SetPadDigitsCycle = _swig_new_instance_method(_datacollection.DataCollection_SetPadDigitsCycle)

    def SetPadDigitsRank(self, digits):
        r"""SetPadDigitsRank(DataCollection self, int digits)"""
        return _datacollection.DataCollection_SetPadDigitsRank(self, digits)
    SetPadDigitsRank = _swig_new_instance_method(_datacollection.DataCollection_SetPadDigitsRank)

    def SetFormat(self, fmt):
        r"""SetFormat(DataCollection self, int fmt)"""
        return _datacollection.DataCollection_SetFormat(self, fmt)
    SetFormat = _swig_new_instance_method(_datacollection.DataCollection_SetFormat)

    def SetCompression(self, comp):
        r"""SetCompression(DataCollection self, bool comp)"""
        return _datacollection.DataCollection_SetCompression(self, comp)
    SetCompression = _swig_new_instance_method(_datacollection.DataCollection_SetCompression)

    def SetPrefixPath(self, prefix):
        r"""SetPrefixPath(DataCollection self, std::string const & prefix)"""
        return _datacollection.DataCollection_SetPrefixPath(self, prefix)
    SetPrefixPath = _swig_new_instance_method(_datacollection.DataCollection_SetPrefixPath)

    def GetPrefixPath(self):
        r"""GetPrefixPath(DataCollection self) -> std::string const &"""
        return _datacollection.DataCollection_GetPrefixPath(self)
    GetPrefixPath = _swig_new_instance_method(_datacollection.DataCollection_GetPrefixPath)

    def Save(self):
        r"""Save(DataCollection self)"""
        return _datacollection.DataCollection_Save(self)
    Save = _swig_new_instance_method(_datacollection.DataCollection_Save)

    def SaveMesh(self):
        r"""SaveMesh(DataCollection self)"""
        return _datacollection.DataCollection_SaveMesh(self)
    SaveMesh = _swig_new_instance_method(_datacollection.DataCollection_SaveMesh)

    def SaveField(self, field_name):
        r"""SaveField(DataCollection self, std::string const & field_name)"""
        return _datacollection.DataCollection_SaveField(self, field_name)
    SaveField = _swig_new_instance_method(_datacollection.DataCollection_SaveField)

    def SaveQField(self, q_field_name):
        r"""SaveQField(DataCollection self, std::string const & q_field_name)"""
        return _datacollection.DataCollection_SaveQField(self, q_field_name)
    SaveQField = _swig_new_instance_method(_datacollection.DataCollection_SaveQField)

    def Load(self, cycle_=0):
        r"""Load(DataCollection self, int cycle_=0)"""
        return _datacollection.DataCollection_Load(self, cycle_)
    Load = _swig_new_instance_method(_datacollection.DataCollection_Load)
    __swig_destroy__ = _datacollection.delete_DataCollection
    NO_ERROR = _datacollection.DataCollection_NO_ERROR
    
    READ_ERROR = _datacollection.DataCollection_READ_ERROR
    
    WRITE_ERROR = _datacollection.DataCollection_WRITE_ERROR
    

    def Error(self):
        r"""Error(DataCollection self) -> int"""
        return _datacollection.DataCollection_Error(self)
    Error = _swig_new_instance_method(_datacollection.DataCollection_Error)

    def ResetError(self, *args, **kwargs):
        r"""ResetError(DataCollection self, int err=NO_ERROR)"""
        return _datacollection.DataCollection_ResetError(self, *args, **kwargs)
    ResetError = _swig_new_instance_method(_datacollection.DataCollection_ResetError)

# Register DataCollection in _datacollection:
_datacollection.DataCollection_swigregister(DataCollection)

class VisItFieldInfo(object):
    r"""Proxy of C++ mfem::VisItFieldInfo class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    association = property(_datacollection.VisItFieldInfo_association_get, _datacollection.VisItFieldInfo_association_set, doc=r"""association : std::string""")
    num_components = property(_datacollection.VisItFieldInfo_num_components_get, _datacollection.VisItFieldInfo_num_components_set, doc=r"""num_components : int""")
    lod = property(_datacollection.VisItFieldInfo_lod_get, _datacollection.VisItFieldInfo_lod_set, doc=r"""lod : int""")

    def __init__(self, *args):
        r"""
        __init__(VisItFieldInfo self) -> VisItFieldInfo
        __init__(VisItFieldInfo self, std::string _association, int _num_components, int _lod=1) -> VisItFieldInfo
        """
        _datacollection.VisItFieldInfo_swiginit(self, _datacollection.new_VisItFieldInfo(*args))
    __swig_destroy__ = _datacollection.delete_VisItFieldInfo

# Register VisItFieldInfo in _datacollection:
_datacollection.VisItFieldInfo_swigregister(VisItFieldInfo)

class VisItDataCollection(DataCollection):
    r"""Proxy of C++ mfem::VisItDataCollection class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, collection_name, mesh_=None):
        r"""__init__(VisItDataCollection self, std::string const & collection_name, Mesh mesh_=None) -> VisItDataCollection"""
        _datacollection.VisItDataCollection_swiginit(self, _datacollection.new_VisItDataCollection(collection_name, mesh_))

    def SetMesh(self, new_mesh):
        r"""SetMesh(VisItDataCollection self, Mesh new_mesh)"""
        return _datacollection.VisItDataCollection_SetMesh(self, new_mesh)
    SetMesh = _swig_new_instance_method(_datacollection.VisItDataCollection_SetMesh)

    def RegisterField(self, field_name, gf):
        r"""RegisterField(VisItDataCollection self, std::string const & field_name, GridFunction gf)"""
        return _datacollection.VisItDataCollection_RegisterField(self, field_name, gf)
    RegisterField = _swig_new_instance_method(_datacollection.VisItDataCollection_RegisterField)

    def RegisterQField(self, q_field_name, qf):
        r"""RegisterQField(VisItDataCollection self, std::string const & q_field_name, QuadratureFunction qf)"""
        return _datacollection.VisItDataCollection_RegisterQField(self, q_field_name, qf)
    RegisterQField = _swig_new_instance_method(_datacollection.VisItDataCollection_RegisterQField)

    def SetLevelsOfDetail(self, levels_of_detail):
        r"""SetLevelsOfDetail(VisItDataCollection self, int levels_of_detail)"""
        return _datacollection.VisItDataCollection_SetLevelsOfDetail(self, levels_of_detail)
    SetLevelsOfDetail = _swig_new_instance_method(_datacollection.VisItDataCollection_SetLevelsOfDetail)

    def SetMaxLevelsOfDetail(self, max_levels_of_detail):
        r"""SetMaxLevelsOfDetail(VisItDataCollection self, int max_levels_of_detail)"""
        return _datacollection.VisItDataCollection_SetMaxLevelsOfDetail(self, max_levels_of_detail)
    SetMaxLevelsOfDetail = _swig_new_instance_method(_datacollection.VisItDataCollection_SetMaxLevelsOfDetail)

    def DeleteAll(self):
        r"""DeleteAll(VisItDataCollection self)"""
        return _datacollection.VisItDataCollection_DeleteAll(self)
    DeleteAll = _swig_new_instance_method(_datacollection.VisItDataCollection_DeleteAll)

    def Save(self):
        r"""Save(VisItDataCollection self)"""
        return _datacollection.VisItDataCollection_Save(self)
    Save = _swig_new_instance_method(_datacollection.VisItDataCollection_Save)

    def SaveRootFile(self):
        r"""SaveRootFile(VisItDataCollection self)"""
        return _datacollection.VisItDataCollection_SaveRootFile(self)
    SaveRootFile = _swig_new_instance_method(_datacollection.VisItDataCollection_SaveRootFile)

    def Load(self, cycle_=0):
        r"""Load(VisItDataCollection self, int cycle_=0)"""
        return _datacollection.VisItDataCollection_Load(self, cycle_)
    Load = _swig_new_instance_method(_datacollection.VisItDataCollection_Load)
    __swig_destroy__ = _datacollection.delete_VisItDataCollection

# Register VisItDataCollection in _datacollection:
_datacollection.VisItDataCollection_swigregister(VisItDataCollection)

class ParaViewDataCollection(DataCollection):
    r"""Proxy of C++ mfem::ParaViewDataCollection class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, collection_name, mesh_=None):
        r"""__init__(ParaViewDataCollection self, std::string const & collection_name, Mesh mesh_=None) -> ParaViewDataCollection"""
        _datacollection.ParaViewDataCollection_swiginit(self, _datacollection.new_ParaViewDataCollection(collection_name, mesh_))

    def SetLevelsOfDetail(self, levels_of_detail_):
        r"""SetLevelsOfDetail(ParaViewDataCollection self, int levels_of_detail_)"""
        return _datacollection.ParaViewDataCollection_SetLevelsOfDetail(self, levels_of_detail_)
    SetLevelsOfDetail = _swig_new_instance_method(_datacollection.ParaViewDataCollection_SetLevelsOfDetail)

    def Save(self):
        r"""Save(ParaViewDataCollection self)"""
        return _datacollection.ParaViewDataCollection_Save(self)
    Save = _swig_new_instance_method(_datacollection.ParaViewDataCollection_Save)

    def SetDataFormat(self, fmt):
        r"""SetDataFormat(ParaViewDataCollection self, mfem::VTKFormat fmt)"""
        return _datacollection.ParaViewDataCollection_SetDataFormat(self, fmt)
    SetDataFormat = _swig_new_instance_method(_datacollection.ParaViewDataCollection_SetDataFormat)

    def SetCompressionLevel(self, compression_level_):
        r"""SetCompressionLevel(ParaViewDataCollection self, int compression_level_)"""
        return _datacollection.ParaViewDataCollection_SetCompressionLevel(self, compression_level_)
    SetCompressionLevel = _swig_new_instance_method(_datacollection.ParaViewDataCollection_SetCompressionLevel)

    def SetCompression(self, compression_):
        r"""SetCompression(ParaViewDataCollection self, bool compression_)"""
        return _datacollection.ParaViewDataCollection_SetCompression(self, compression_)
    SetCompression = _swig_new_instance_method(_datacollection.ParaViewDataCollection_SetCompression)

    def IsBinaryFormat(self):
        r"""IsBinaryFormat(ParaViewDataCollection self) -> bool"""
        return _datacollection.ParaViewDataCollection_IsBinaryFormat(self)
    IsBinaryFormat = _swig_new_instance_method(_datacollection.ParaViewDataCollection_IsBinaryFormat)

    def SetHighOrderOutput(self, high_order_output_):
        r"""SetHighOrderOutput(ParaViewDataCollection self, bool high_order_output_)"""
        return _datacollection.ParaViewDataCollection_SetHighOrderOutput(self, high_order_output_)
    SetHighOrderOutput = _swig_new_instance_method(_datacollection.ParaViewDataCollection_SetHighOrderOutput)

    def Load(self, cycle_=0):
        r"""Load(ParaViewDataCollection self, int cycle_=0)"""
        return _datacollection.ParaViewDataCollection_Load(self, cycle_)
    Load = _swig_new_instance_method(_datacollection.ParaViewDataCollection_Load)
    __swig_destroy__ = _datacollection.delete_ParaViewDataCollection

# Register ParaViewDataCollection in _datacollection:
_datacollection.ParaViewDataCollection_swigregister(ParaViewDataCollection)






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _intrules
else:
    import _intrules

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _intrules.SWIG_PyInstanceMethod_New
_swig_new_static_method = _intrules.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import mfem._par.array
import mfem._par.mem_manager
class IntegrationPointArray(object):
    r"""Proxy of C++ mfem::Array< mfem::IntegrationPoint > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(IntegrationPointArray self) -> IntegrationPointArray
        __init__(IntegrationPointArray self, mfem::MemoryType mt) -> IntegrationPointArray
        __init__(IntegrationPointArray self, int asize) -> IntegrationPointArray
        __init__(IntegrationPointArray self, IntegrationPoint _data, int asize) -> IntegrationPointArray
        __init__(IntegrationPointArray self, IntegrationPointArray src) -> IntegrationPointArray
        """
        _intrules.IntegrationPointArray_swiginit(self, _intrules.new_IntegrationPointArray(*args))

        if len(args) == 1 and isinstance(args[0], list):
            self.MakeDataOwner()



    __swig_destroy__ = _intrules.delete_IntegrationPointArray

    def GetData(self, *args):
        r"""
        GetData(IntegrationPointArray self) -> IntegrationPoint
        GetData(IntegrationPointArray self) -> IntegrationPoint
        """
        return _intrules.IntegrationPointArray_GetData(self, *args)
    GetData = _swig_new_instance_method(_intrules.IntegrationPointArray_GetData)

    def GetMemory(self, *args):
        r"""
        GetMemory(IntegrationPointArray self) -> mfem::Memory< mfem::IntegrationPoint >
        GetMemory(IntegrationPointArray self) -> mfem::Memory< mfem::IntegrationPoint > const &
        """
        return _intrules.IntegrationPointArray_GetMemory(self, *args)
    GetMemory = _swig_new_instance_method(_intrules.IntegrationPointArray_GetMemory)

    def UseDevice(self):
        r"""UseDevice(IntegrationPointArray self) -> bool"""
        return _intrules.IntegrationPointArray_UseDevice(self)
    UseDevice = _swig_new_instance_method(_intrules.IntegrationPointArray_UseDevice)

    def OwnsData(self):
        r"""OwnsData(IntegrationPointArray self) -> bool"""
        return _intrules.IntegrationPointArray_OwnsData(self)
    OwnsData = _swig_new_instance_method(_intrules.IntegrationPointArray_OwnsData)

    def StealData(self, p):
        r"""StealData(IntegrationPointArray self, mfem::IntegrationPoint ** p)"""
        return _intrules.IntegrationPointArray_StealData(self, p)
    StealData = _swig_new_instance_method(_intrules.IntegrationPointArray_StealData)

    def LoseData(self):
        r"""LoseData(IntegrationPointArray self)"""
        return _intrules.IntegrationPointArray_LoseData(self)
    LoseData = _swig_new_instance_method(_intrules.IntegrationPointArray_LoseData)

    def MakeDataOwner(self):
        r"""MakeDataOwner(IntegrationPointArray self)"""
        return _intrules.IntegrationPointArray_MakeDataOwner(self)
    MakeDataOwner = _swig_new_instance_method(_intrules.IntegrationPointArray_MakeDataOwner)

    def Size(self):
        r"""Size(IntegrationPointArray self) -> int"""
        return _intrules.IntegrationPointArray_Size(self)
    Size = _swig_new_instance_method(_intrules.IntegrationPointArray_Size)

    def SetSize(self, *args):
        r"""
        SetSize(IntegrationPointArray self, int nsize)
        SetSize(IntegrationPointArray self, int nsize, IntegrationPoint initval)
        SetSize(IntegrationPointArray self, int nsize, mfem::MemoryType mt)
        """
        return _intrules.IntegrationPointArray_SetSize(self, *args)
    SetSize = _swig_new_instance_method(_intrules.IntegrationPointArray_SetSize)

    def Capacity(self):
        r"""Capacity(IntegrationPointArray self) -> int"""
        return _intrules.IntegrationPointArray_Capacity(self)
    Capacity = _swig_new_instance_method(_intrules.IntegrationPointArray_Capacity)

    def Reserve(self, capacity):
        r"""Reserve(IntegrationPointArray self, int capacity)"""
        return _intrules.IntegrationPointArray_Reserve(self, capacity)
    Reserve = _swig_new_instance_method(_intrules.IntegrationPointArray_Reserve)

    def Append(self, *args):
        r"""
        Append(IntegrationPointArray self, IntegrationPoint el) -> int
        Append(IntegrationPointArray self, IntegrationPoint els, int nels) -> int
        Append(IntegrationPointArray self, IntegrationPointArray els) -> int
        """
        return _intrules.IntegrationPointArray_Append(self, *args)
    Append = _swig_new_instance_method(_intrules.IntegrationPointArray_Append)

    def Prepend(self, el):
        r"""Prepend(IntegrationPointArray self, IntegrationPoint el) -> int"""
        return _intrules.IntegrationPointArray_Prepend(self, el)
    Prepend = _swig_new_instance_method(_intrules.IntegrationPointArray_Prepend)

    def Last(self, *args):
        r"""
        Last(IntegrationPointArray self) -> IntegrationPoint
        Last(IntegrationPointArray self) -> IntegrationPoint
        """
        return _intrules.IntegrationPointArray_Last(self, *args)
    Last = _swig_new_instance_method(_intrules.IntegrationPointArray_Last)

    def DeleteLast(self):
        r"""DeleteLast(IntegrationPointArray self)"""
        return _intrules.IntegrationPointArray_DeleteLast(self)
    DeleteLast = _swig_new_instance_method(_intrules.IntegrationPointArray_DeleteLast)

    def DeleteAll(self):
        r"""DeleteAll(IntegrationPointArray self)"""
        return _intrules.IntegrationPointArray_DeleteAll(self)
    DeleteAll = _swig_new_instance_method(_intrules.IntegrationPointArray_DeleteAll)

    def Copy(self, copy):
        r"""Copy(IntegrationPointArray self, IntegrationPointArray copy)"""
        return _intrules.IntegrationPointArray_Copy(self, copy)
    Copy = _swig_new_instance_method(_intrules.IntegrationPointArray_Copy)

    def MakeRef(self, *args):
        r"""
        MakeRef(IntegrationPointArray self, IntegrationPoint arg2, int arg3)
        MakeRef(IntegrationPointArray self, IntegrationPointArray master)
        """
        return _intrules.IntegrationPointArray_MakeRef(self, *args)
    MakeRef = _swig_new_instance_method(_intrules.IntegrationPointArray_MakeRef)

    def GetSubArray(self, offset, sa_size, sa):
        r"""GetSubArray(IntegrationPointArray self, int offset, int sa_size, IntegrationPointArray sa)"""
        return _intrules.IntegrationPointArray_GetSubArray(self, offset, sa_size, sa)
    GetSubArray = _swig_new_instance_method(_intrules.IntegrationPointArray_GetSubArray)

    def begin(self, *args):
        r"""
        begin(IntegrationPointArray self) -> IntegrationPoint
        begin(IntegrationPointArray self) -> IntegrationPoint
        """
        return _intrules.IntegrationPointArray_begin(self, *args)
    begin = _swig_new_instance_method(_intrules.IntegrationPointArray_begin)

    def end(self, *args):
        r"""
        end(IntegrationPointArray self) -> IntegrationPoint
        end(IntegrationPointArray self) -> IntegrationPoint
        """
        return _intrules.IntegrationPointArray_end(self, *args)
    end = _swig_new_instance_method(_intrules.IntegrationPointArray_end)

    def MemoryUsage(self):
        r"""MemoryUsage(IntegrationPointArray self) -> long"""
        return _intrules.IntegrationPointArray_MemoryUsage(self)
    MemoryUsage = _swig_new_instance_method(_intrules.IntegrationPointArray_MemoryUsage)

    def Read(self, on_dev=True):
        r"""Read(IntegrationPointArray self, bool on_dev=True) -> IntegrationPoint"""
        return _intrules.IntegrationPointArray_Read(self, on_dev)
    Read = _swig_new_instance_method(_intrules.IntegrationPointArray_Read)

    def HostRead(self):
        r"""HostRead(IntegrationPointArray self) -> IntegrationPoint"""
        return _intrules.IntegrationPointArray_HostRead(self)
    HostRead = _swig_new_instance_method(_intrules.IntegrationPointArray_HostRead)

    def Write(self, on_dev=True):
        r"""Write(IntegrationPointArray self, bool on_dev=True) -> IntegrationPoint"""
        return _intrules.IntegrationPointArray_Write(self, on_dev)
    Write = _swig_new_instance_method(_intrules.IntegrationPointArray_Write)

    def HostWrite(self):
        r"""HostWrite(IntegrationPointArray self) -> IntegrationPoint"""
        return _intrules.IntegrationPointArray_HostWrite(self)
    HostWrite = _swig_new_instance_method(_intrules.IntegrationPointArray_HostWrite)

    def ReadWrite(self, on_dev=True):
        r"""ReadWrite(IntegrationPointArray self, bool on_dev=True) -> IntegrationPoint"""
        return _intrules.IntegrationPointArray_ReadWrite(self, on_dev)
    ReadWrite = _swig_new_instance_method(_intrules.IntegrationPointArray_ReadWrite)

    def HostReadWrite(self):
        r"""HostReadWrite(IntegrationPointArray self) -> IntegrationPoint"""
        return _intrules.IntegrationPointArray_HostReadWrite(self)
    HostReadWrite = _swig_new_instance_method(_intrules.IntegrationPointArray_HostReadWrite)

    def __setitem__(self, i, v):
        r"""__setitem__(IntegrationPointArray self, int i, IntegrationPoint v)"""
        return _intrules.IntegrationPointArray___setitem__(self, i, v)
    __setitem__ = _swig_new_instance_method(_intrules.IntegrationPointArray___setitem__)

    def __getitem__(self, i):
        r"""__getitem__(IntegrationPointArray self, int const i) -> IntegrationPoint"""
        return _intrules.IntegrationPointArray___getitem__(self, i)
    __getitem__ = _swig_new_instance_method(_intrules.IntegrationPointArray___getitem__)

    def Assign(self, *args):
        r"""
        Assign(IntegrationPointArray self, IntegrationPoint arg2)
        Assign(IntegrationPointArray self, IntegrationPoint a)
        """
        return _intrules.IntegrationPointArray_Assign(self, *args)
    Assign = _swig_new_instance_method(_intrules.IntegrationPointArray_Assign)

    def ToList(self):
        return [self[i] for i in range(self.Size())]



# Register IntegrationPointArray in _intrules:
_intrules.IntegrationPointArray_swigregister(IntegrationPointArray)

class IntegrationPoint(object):
    r"""Proxy of C++ mfem::IntegrationPoint class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    x = property(_intrules.IntegrationPoint_x_get, _intrules.IntegrationPoint_x_set, doc=r"""x : double""")
    y = property(_intrules.IntegrationPoint_y_get, _intrules.IntegrationPoint_y_set, doc=r"""y : double""")
    z = property(_intrules.IntegrationPoint_z_get, _intrules.IntegrationPoint_z_set, doc=r"""z : double""")
    weight = property(_intrules.IntegrationPoint_weight_get, _intrules.IntegrationPoint_weight_set, doc=r"""weight : double""")
    index = property(_intrules.IntegrationPoint_index_get, _intrules.IntegrationPoint_index_set, doc=r"""index : int""")

    def Init(self, i):
        r"""Init(IntegrationPoint self, int const i)"""
        return _intrules.IntegrationPoint_Init(self, i)
    Init = _swig_new_instance_method(_intrules.IntegrationPoint_Init)

    def Get(self, p, dim):
        r"""Get(IntegrationPoint self, double * p, int const dim)"""
        return _intrules.IntegrationPoint_Get(self, p, dim)
    Get = _swig_new_instance_method(_intrules.IntegrationPoint_Get)

    def Set(self, *args):
        r"""
        Set(IntegrationPoint self, double const * p, int const dim)
        Set(IntegrationPoint self, double const x1, double const x2, double const x3, double const w)
        """
        return _intrules.IntegrationPoint_Set(self, *args)
    Set = _swig_new_instance_method(_intrules.IntegrationPoint_Set)

    def Set3w(self, p):
        r"""Set3w(IntegrationPoint self, double const * p)"""
        return _intrules.IntegrationPoint_Set3w(self, p)
    Set3w = _swig_new_instance_method(_intrules.IntegrationPoint_Set3w)

    def Set3(self, *args):
        r"""
        Set3(IntegrationPoint self, double const x1, double const x2, double const x3)
        Set3(IntegrationPoint self, double const * p)
        """
        return _intrules.IntegrationPoint_Set3(self, *args)
    Set3 = _swig_new_instance_method(_intrules.IntegrationPoint_Set3)

    def Set2w(self, *args):
        r"""
        Set2w(IntegrationPoint self, double const x1, double const x2, double const w)
        Set2w(IntegrationPoint self, double const * p)
        """
        return _intrules.IntegrationPoint_Set2w(self, *args)
    Set2w = _swig_new_instance_method(_intrules.IntegrationPoint_Set2w)

    def Set2(self, *args):
        r"""
        Set2(IntegrationPoint self, double const x1, double const x2)
        Set2(IntegrationPoint self, double const * p)
        """
        return _intrules.IntegrationPoint_Set2(self, *args)
    Set2 = _swig_new_instance_method(_intrules.IntegrationPoint_Set2)

    def Set1w(self, *args):
        r"""
        Set1w(IntegrationPoint self, double const x1, double const w)
        Set1w(IntegrationPoint self, double const * p)
        """
        return _intrules.IntegrationPoint_Set1w(self, *args)
    Set1w = _swig_new_instance_method(_intrules.IntegrationPoint_Set1w)

    def __init__(self):
        r"""__init__(IntegrationPoint self) -> IntegrationPoint"""
        _intrules.IntegrationPoint_swiginit(self, _intrules.new_IntegrationPoint())
    __swig_destroy__ = _intrules.delete_IntegrationPoint

# Register IntegrationPoint in _intrules:
_intrules.IntegrationPoint_swigregister(IntegrationPoint)

class IntegrationRule(IntegrationPointArray):
    r"""Proxy of C++ mfem::IntegrationRule class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(IntegrationRule self) -> IntegrationRule
        __init__(IntegrationRule self, int NP) -> IntegrationRule
        __init__(IntegrationRule self, IntegrationRule irx, IntegrationRule iry) -> IntegrationRule
        __init__(IntegrationRule self, IntegrationRule irx, IntegrationRule iry, IntegrationRule irz) -> IntegrationRule
        """
        _intrules.IntegrationRule_swiginit(self, _intrules.new_IntegrationRule(*args))

    def GetOrder(self):
        r"""GetOrder(IntegrationRule self) -> int"""
        return _intrules.IntegrationRule_GetOrder(self)
    GetOrder = _swig_new_instance_method(_intrules.IntegrationRule_GetOrder)

    def SetOrder(self, order):
        r"""SetOrder(IntegrationRule self, int const order)"""
        return _intrules.IntegrationRule_SetOrder(self, order)
    SetOrder = _swig_new_instance_method(_intrules.IntegrationRule_SetOrder)

    def GetNPoints(self):
        r"""GetNPoints(IntegrationRule self) -> int"""
        return _intrules.IntegrationRule_GetNPoints(self)
    GetNPoints = _swig_new_instance_method(_intrules.IntegrationRule_GetNPoints)

    def IntPoint(self, *args):
        r"""
        IntPoint(IntegrationRule self, int i) -> IntegrationPoint
        IntPoint(IntegrationRule self, int i) -> IntegrationPoint
        """
        return _intrules.IntegrationRule_IntPoint(self, *args)
    IntPoint = _swig_new_instance_method(_intrules.IntegrationRule_IntPoint)

    def GetWeights(self):
        r"""GetWeights(IntegrationRule self) -> doubleArray"""
        return _intrules.IntegrationRule_GetWeights(self)
    GetWeights = _swig_new_instance_method(_intrules.IntegrationRule_GetWeights)
    __swig_destroy__ = _intrules.delete_IntegrationRule

# Register IntegrationRule in _intrules:
_intrules.IntegrationRule_swigregister(IntegrationRule)

class QuadratureFunctions1D(object):
    r"""Proxy of C++ mfem::QuadratureFunctions1D class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def GaussLegendre(self, np, ir):
        r"""GaussLegendre(QuadratureFunctions1D self, int const np, IntegrationRule ir)"""
        return _intrules.QuadratureFunctions1D_GaussLegendre(self, np, ir)
    GaussLegendre = _swig_new_instance_method(_intrules.QuadratureFunctions1D_GaussLegendre)

    def GaussLobatto(self, np, ir):
        r"""GaussLobatto(QuadratureFunctions1D self, int const np, IntegrationRule ir)"""
        return _intrules.QuadratureFunctions1D_GaussLobatto(self, np, ir)
    GaussLobatto = _swig_new_instance_method(_intrules.QuadratureFunctions1D_GaussLobatto)

    def OpenUniform(self, np, ir):
        r"""OpenUniform(QuadratureFunctions1D self, int const np, IntegrationRule ir)"""
        return _intrules.QuadratureFunctions1D_OpenUniform(self, np, ir)
    OpenUniform = _swig_new_instance_method(_intrules.QuadratureFunctions1D_OpenUniform)

    def ClosedUniform(self, np, ir):
        r"""ClosedUniform(QuadratureFunctions1D self, int const np, IntegrationRule ir)"""
        return _intrules.QuadratureFunctions1D_ClosedUniform(self, np, ir)
    ClosedUniform = _swig_new_instance_method(_intrules.QuadratureFunctions1D_ClosedUniform)

    def OpenHalfUniform(self, np, ir):
        r"""OpenHalfUniform(QuadratureFunctions1D self, int const np, IntegrationRule ir)"""
        return _intrules.QuadratureFunctions1D_OpenHalfUniform(self, np, ir)
    OpenHalfUniform = _swig_new_instance_method(_intrules.QuadratureFunctions1D_OpenHalfUniform)

    def ClosedGL(self, np, ir):
        r"""ClosedGL(QuadratureFunctions1D self, int const np, IntegrationRule ir)"""
        return _intrules.QuadratureFunctions1D_ClosedGL(self, np, ir)
    ClosedGL = _swig_new_instance_method(_intrules.QuadratureFunctions1D_ClosedGL)

    def GivePolyPoints(self, np, pts, type):
        r"""GivePolyPoints(QuadratureFunctions1D self, int const np, double * pts, int const type)"""
        return _intrules.QuadratureFunctions1D_GivePolyPoints(self, np, pts, type)
    GivePolyPoints = _swig_new_instance_method(_intrules.QuadratureFunctions1D_GivePolyPoints)

    def __init__(self):
        r"""__init__(QuadratureFunctions1D self) -> QuadratureFunctions1D"""
        _intrules.QuadratureFunctions1D_swiginit(self, _intrules.new_QuadratureFunctions1D())
    __swig_destroy__ = _intrules.delete_QuadratureFunctions1D

# Register QuadratureFunctions1D in _intrules:
_intrules.QuadratureFunctions1D_swigregister(QuadratureFunctions1D)

class Quadrature1D(object):
    r"""Proxy of C++ mfem::Quadrature1D class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    Invalid = _intrules.Quadrature1D_Invalid
    
    GaussLegendre = _intrules.Quadrature1D_GaussLegendre
    
    GaussLobatto = _intrules.Quadrature1D_GaussLobatto
    
    OpenUniform = _intrules.Quadrature1D_OpenUniform
    
    ClosedUniform = _intrules.Quadrature1D_ClosedUniform
    
    OpenHalfUniform = _intrules.Quadrature1D_OpenHalfUniform
    
    ClosedGL = _intrules.Quadrature1D_ClosedGL
    

    @staticmethod
    def CheckClosed(type):
        r"""CheckClosed(int type) -> int"""
        return _intrules.Quadrature1D_CheckClosed(type)
    CheckClosed = _swig_new_static_method(_intrules.Quadrature1D_CheckClosed)

    @staticmethod
    def CheckOpen(type):
        r"""CheckOpen(int type) -> int"""
        return _intrules.Quadrature1D_CheckOpen(type)
    CheckOpen = _swig_new_static_method(_intrules.Quadrature1D_CheckOpen)

    def __init__(self):
        r"""__init__(Quadrature1D self) -> Quadrature1D"""
        _intrules.Quadrature1D_swiginit(self, _intrules.new_Quadrature1D())
    __swig_destroy__ = _intrules.delete_Quadrature1D

# Register Quadrature1D in _intrules:
_intrules.Quadrature1D_swigregister(Quadrature1D)

def Quadrature1D_CheckClosed(type):
    r"""Quadrature1D_CheckClosed(int type) -> int"""
    return _intrules.Quadrature1D_CheckClosed(type)
Quadrature1D_CheckClosed = _intrules.Quadrature1D_CheckClosed

def Quadrature1D_CheckOpen(type):
    r"""Quadrature1D_CheckOpen(int type) -> int"""
    return _intrules.Quadrature1D_CheckOpen(type)
Quadrature1D_CheckOpen = _intrules.Quadrature1D_CheckOpen

class IntegrationRules(object):
    r"""Proxy of C++ mfem::IntegrationRules class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args, **kwargs):
        r"""__init__(IntegrationRules self, int Ref=0, int type=GaussLegendre) -> IntegrationRules"""
        _intrules.IntegrationRules_swiginit(self, _intrules.new_IntegrationRules(*args, **kwargs))

    def Get(self, GeomType, Order):
        r"""Get(IntegrationRules self, int GeomType, int Order) -> IntegrationRule"""
        return _intrules.IntegrationRules_Get(self, GeomType, Order)
    Get = _swig_new_instance_method(_intrules.IntegrationRules_Get)

    def Set(self, GeomType, Order, IntRule):
        r"""Set(IntegrationRules self, int GeomType, int Order, IntegrationRule IntRule)"""
        return _intrules.IntegrationRules_Set(self, GeomType, Order, IntRule)
    Set = _swig_new_instance_method(_intrules.IntegrationRules_Set)

    def SetOwnRules(self, o):
        r"""SetOwnRules(IntegrationRules self, int o)"""
        return _intrules.IntegrationRules_SetOwnRules(self, o)
    SetOwnRules = _swig_new_instance_method(_intrules.IntegrationRules_SetOwnRules)
    __swig_destroy__ = _intrules.delete_IntegrationRules

# Register IntegrationRules in _intrules:
_intrules.IntegrationRules_swigregister(IntegrationRules)


cvar = _intrules.cvar
IntRules = cvar.IntRules
RefinedIntRules = cvar.RefinedIntRules




# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _handle
else:
    import _handle

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _handle.SWIG_PyInstanceMethod_New
_swig_new_static_method = _handle.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

MFEM_VERSION = _handle.MFEM_VERSION

MFEM_VERSION_STRING = _handle.MFEM_VERSION_STRING

MFEM_VERSION_TYPE = _handle.MFEM_VERSION_TYPE

MFEM_VERSION_TYPE_RELEASE = _handle.MFEM_VERSION_TYPE_RELEASE

MFEM_VERSION_TYPE_DEVELOPMENT = _handle.MFEM_VERSION_TYPE_DEVELOPMENT

MFEM_VERSION_MAJOR = _handle.MFEM_VERSION_MAJOR

MFEM_VERSION_MINOR = _handle.MFEM_VERSION_MINOR

MFEM_VERSION_PATCH = _handle.MFEM_VERSION_PATCH

import mfem._ser.operators
import mfem._ser.mem_manager
import mfem._ser.vector
import mfem._ser.array
class OperatorHandle(object):
    r"""Proxy of C++ mfem::OperatorHandle class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(OperatorHandle self) -> OperatorHandle
        __init__(OperatorHandle self, mfem::Operator::Type tid) -> OperatorHandle
        """
        _handle.OperatorHandle_swiginit(self, _handle.new_OperatorHandle(*args))
    __swig_destroy__ = _handle.delete_OperatorHandle

    def Ptr(self):
        r"""Ptr(OperatorHandle self) -> Operator"""
        return _handle.OperatorHandle_Ptr(self)
    Ptr = _swig_new_instance_method(_handle.OperatorHandle_Ptr)

    def __deref__(self):
        r"""__deref__(OperatorHandle self) -> Operator"""
        return _handle.OperatorHandle___deref__(self)
    __deref__ = _swig_new_instance_method(_handle.OperatorHandle___deref__)

    def __ref__(self):
        r"""__ref__(OperatorHandle self) -> Operator"""
        return _handle.OperatorHandle___ref__(self)
    __ref__ = _swig_new_instance_method(_handle.OperatorHandle___ref__)

    def Type(self):
        r"""Type(OperatorHandle self) -> mfem::Operator::Type"""
        return _handle.OperatorHandle_Type(self)
    Type = _swig_new_instance_method(_handle.OperatorHandle_Type)

    def OwnsOperator(self):
        r"""OwnsOperator(OperatorHandle self) -> bool"""
        return _handle.OperatorHandle_OwnsOperator(self)
    OwnsOperator = _swig_new_instance_method(_handle.OperatorHandle_OwnsOperator)

    def SetOperatorOwner(self, own=True):
        r"""SetOperatorOwner(OperatorHandle self, bool own=True)"""
        return _handle.OperatorHandle_SetOperatorOwner(self, own)
    SetOperatorOwner = _swig_new_instance_method(_handle.OperatorHandle_SetOperatorOwner)

    def Clear(self):
        r"""Clear(OperatorHandle self)"""
        return _handle.OperatorHandle_Clear(self)
    Clear = _swig_new_instance_method(_handle.OperatorHandle_Clear)

    def SetType(self, tid):
        r"""SetType(OperatorHandle self, mfem::Operator::Type tid)"""
        return _handle.OperatorHandle_SetType(self, tid)
    SetType = _swig_new_instance_method(_handle.OperatorHandle_SetType)

    def MakePtAP(self, A, P):
        r"""MakePtAP(OperatorHandle self, OperatorHandle A, OperatorHandle P)"""
        return _handle.OperatorHandle_MakePtAP(self, A, P)
    MakePtAP = _swig_new_instance_method(_handle.OperatorHandle_MakePtAP)

    def MakeRAP(self, Rt, A, P):
        r"""MakeRAP(OperatorHandle self, OperatorHandle Rt, OperatorHandle A, OperatorHandle P)"""
        return _handle.OperatorHandle_MakeRAP(self, Rt, A, P)
    MakeRAP = _swig_new_instance_method(_handle.OperatorHandle_MakeRAP)

    def EliminateRowsCols(self, A, ess_dof_list):
        r"""EliminateRowsCols(OperatorHandle self, OperatorHandle A, intArray ess_dof_list)"""
        return _handle.OperatorHandle_EliminateRowsCols(self, A, ess_dof_list)
    EliminateRowsCols = _swig_new_instance_method(_handle.OperatorHandle_EliminateRowsCols)

    def EliminateRows(self, ess_dof_list):
        r"""EliminateRows(OperatorHandle self, intArray ess_dof_list)"""
        return _handle.OperatorHandle_EliminateRows(self, ess_dof_list)
    EliminateRows = _swig_new_instance_method(_handle.OperatorHandle_EliminateRows)

    def EliminateCols(self, ess_dof_list):
        r"""EliminateCols(OperatorHandle self, intArray ess_dof_list)"""
        return _handle.OperatorHandle_EliminateCols(self, ess_dof_list)
    EliminateCols = _swig_new_instance_method(_handle.OperatorHandle_EliminateCols)

    def EliminateBC(self, A_e, ess_dof_list, X, B):
        r"""EliminateBC(OperatorHandle self, OperatorHandle A_e, intArray ess_dof_list, Vector X, Vector B)"""
        return _handle.OperatorHandle_EliminateBC(self, A_e, ess_dof_list, X, B)
    EliminateBC = _swig_new_instance_method(_handle.OperatorHandle_EliminateBC)

    def As(self):
        r"""As(OperatorHandle self) -> mfem::SparseMatrix *"""
        return _handle.OperatorHandle_As(self)
    As = _swig_new_instance_method(_handle.OperatorHandle_As)

    def Is(self):
        r"""Is(OperatorHandle self) -> mfem::SparseMatrix *"""
        return _handle.OperatorHandle_Is(self)
    Is = _swig_new_instance_method(_handle.OperatorHandle_Is)

    def Get(self, A):
        r"""Get(OperatorHandle self, mfem::SparseMatrix *& A)"""
        return _handle.OperatorHandle_Get(self, A)
    Get = _swig_new_instance_method(_handle.OperatorHandle_Get)

    def Reset(self, A, own_A=True):
        r"""Reset(OperatorHandle self, mfem::SparseMatrix * A, bool own_A=True)"""
        return _handle.OperatorHandle_Reset(self, A, own_A)
    Reset = _swig_new_instance_method(_handle.OperatorHandle_Reset)

    def ConvertFrom(self, *args):
        r"""
        ConvertFrom(OperatorHandle self, OperatorHandle A)
        ConvertFrom(OperatorHandle self, mfem::SparseMatrix * A)
        """
        return _handle.OperatorHandle_ConvertFrom(self, *args)
    ConvertFrom = _swig_new_instance_method(_handle.OperatorHandle_ConvertFrom)

    def InitTVectors(self, Po, Ri, Pi, x, b, X, B):
        r"""InitTVectors(OperatorHandle self, Operator Po, Operator Ri, Operator Pi, Vector x, Vector b, Vector X, Vector B)"""
        return _handle.OperatorHandle_InitTVectors(self, Po, Ri, Pi, x, b, X, B)
    InitTVectors = _swig_new_instance_method(_handle.OperatorHandle_InitTVectors)

    def Height(self):
        r"""Height(OperatorHandle self) -> int"""
        return _handle.OperatorHandle_Height(self)
    Height = _swig_new_instance_method(_handle.OperatorHandle_Height)

    def NumRows(self):
        r"""NumRows(OperatorHandle self) -> int"""
        return _handle.OperatorHandle_NumRows(self)
    NumRows = _swig_new_instance_method(_handle.OperatorHandle_NumRows)

    def Width(self):
        r"""Width(OperatorHandle self) -> int"""
        return _handle.OperatorHandle_Width(self)
    Width = _swig_new_instance_method(_handle.OperatorHandle_Width)

    def NumCols(self):
        r"""NumCols(OperatorHandle self) -> int"""
        return _handle.OperatorHandle_NumCols(self)
    NumCols = _swig_new_instance_method(_handle.OperatorHandle_NumCols)

    def GetMemoryClass(self):
        r"""GetMemoryClass(OperatorHandle self) -> mfem::MemoryClass"""
        return _handle.OperatorHandle_GetMemoryClass(self)
    GetMemoryClass = _swig_new_instance_method(_handle.OperatorHandle_GetMemoryClass)

    def Mult(self, x, y):
        r"""Mult(OperatorHandle self, Vector x, Vector y)"""
        return _handle.OperatorHandle_Mult(self, x, y)
    Mult = _swig_new_instance_method(_handle.OperatorHandle_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(OperatorHandle self, Vector x, Vector y)"""
        return _handle.OperatorHandle_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_handle.OperatorHandle_MultTranspose)

    def GetGradient(self, x):
        r"""GetGradient(OperatorHandle self, Vector x) -> Operator"""
        return _handle.OperatorHandle_GetGradient(self, x)
    GetGradient = _swig_new_instance_method(_handle.OperatorHandle_GetGradient)

    def GetProlongation(self):
        r"""GetProlongation(OperatorHandle self) -> Operator"""
        return _handle.OperatorHandle_GetProlongation(self)
    GetProlongation = _swig_new_instance_method(_handle.OperatorHandle_GetProlongation)

    def GetRestriction(self):
        r"""GetRestriction(OperatorHandle self) -> Operator"""
        return _handle.OperatorHandle_GetRestriction(self)
    GetRestriction = _swig_new_instance_method(_handle.OperatorHandle_GetRestriction)

    def GetOutputProlongation(self):
        r"""GetOutputProlongation(OperatorHandle self) -> Operator"""
        return _handle.OperatorHandle_GetOutputProlongation(self)
    GetOutputProlongation = _swig_new_instance_method(_handle.OperatorHandle_GetOutputProlongation)

    def GetOutputRestriction(self):
        r"""GetOutputRestriction(OperatorHandle self) -> Operator"""
        return _handle.OperatorHandle_GetOutputRestriction(self)
    GetOutputRestriction = _swig_new_instance_method(_handle.OperatorHandle_GetOutputRestriction)

    def FormLinearSystem(self, ess_tdof_list, x, b, A, X, B, copy_interior=0):
        r"""FormLinearSystem(OperatorHandle self, intArray ess_tdof_list, Vector x, Vector b, mfem::Operator *& A, Vector X, Vector B, int copy_interior=0)"""
        return _handle.OperatorHandle_FormLinearSystem(self, ess_tdof_list, x, b, A, X, B, copy_interior)
    FormLinearSystem = _swig_new_instance_method(_handle.OperatorHandle_FormLinearSystem)

    def FormRectangularLinearSystem(self, trial_tdof_list, test_tdof_list, x, b, A, X, B):
        r"""FormRectangularLinearSystem(OperatorHandle self, intArray trial_tdof_list, intArray test_tdof_list, Vector x, Vector b, mfem::Operator *& A, Vector X, Vector B)"""
        return _handle.OperatorHandle_FormRectangularLinearSystem(self, trial_tdof_list, test_tdof_list, x, b, A, X, B)
    FormRectangularLinearSystem = _swig_new_instance_method(_handle.OperatorHandle_FormRectangularLinearSystem)

    def RecoverFEMSolution(self, X, b, x):
        r"""RecoverFEMSolution(OperatorHandle self, Vector X, Vector b, Vector x)"""
        return _handle.OperatorHandle_RecoverFEMSolution(self, X, b, x)
    RecoverFEMSolution = _swig_new_instance_method(_handle.OperatorHandle_RecoverFEMSolution)

    def FormSystemOperator(self, ess_tdof_list, A):
        r"""FormSystemOperator(OperatorHandle self, intArray ess_tdof_list, mfem::Operator *& A)"""
        return _handle.OperatorHandle_FormSystemOperator(self, ess_tdof_list, A)
    FormSystemOperator = _swig_new_instance_method(_handle.OperatorHandle_FormSystemOperator)

    def FormRectangularSystemOperator(self, trial_tdof_list, test_tdof_list, A):
        r"""FormRectangularSystemOperator(OperatorHandle self, intArray trial_tdof_list, intArray test_tdof_list, mfem::Operator *& A)"""
        return _handle.OperatorHandle_FormRectangularSystemOperator(self, trial_tdof_list, test_tdof_list, A)
    FormRectangularSystemOperator = _swig_new_instance_method(_handle.OperatorHandle_FormRectangularSystemOperator)

    def FormDiscreteOperator(self, A):
        r"""FormDiscreteOperator(OperatorHandle self, mfem::Operator *& A)"""
        return _handle.OperatorHandle_FormDiscreteOperator(self, A)
    FormDiscreteOperator = _swig_new_instance_method(_handle.OperatorHandle_FormDiscreteOperator)

    def PrintMatlab(self, out, n=0, m=0):
        r"""PrintMatlab(OperatorHandle self, std::ostream & out, int n=0, int m=0)"""
        return _handle.OperatorHandle_PrintMatlab(self, out, n, m)
    PrintMatlab = _swig_new_instance_method(_handle.OperatorHandle_PrintMatlab)

    def GetType(self):
        r"""GetType(OperatorHandle self) -> mfem::Operator::Type"""
        return _handle.OperatorHandle_GetType(self)
    GetType = _swig_new_instance_method(_handle.OperatorHandle_GetType)

# Register OperatorHandle in _handle:
_handle.OperatorHandle_swigregister(OperatorHandle)


OperatorPtr=OperatorHandle  






# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _pfespace
else:
    import _pfespace

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

_swig_new_instance_method = _pfespace.SWIG_PyInstanceMethod_New
_swig_new_static_method = _pfespace.SWIG_PyStaticMethod_New

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

MFEM_VERSION = _pfespace.MFEM_VERSION
MFEM_VERSION_STRING = _pfespace.MFEM_VERSION_STRING
MFEM_VERSION_TYPE = _pfespace.MFEM_VERSION_TYPE
MFEM_VERSION_TYPE_RELEASE = _pfespace.MFEM_VERSION_TYPE_RELEASE
MFEM_VERSION_TYPE_DEVELOPMENT = _pfespace.MFEM_VERSION_TYPE_DEVELOPMENT
MFEM_VERSION_MAJOR = _pfespace.MFEM_VERSION_MAJOR
MFEM_VERSION_MINOR = _pfespace.MFEM_VERSION_MINOR
MFEM_VERSION_PATCH = _pfespace.MFEM_VERSION_PATCH
MFEM_HYPRE_VERSION = _pfespace.MFEM_HYPRE_VERSION
import mfem._par.operators
import mfem._par.mem_manager
import mfem._par.vector
import mfem._par.array
import mfem._par.fespace
import mfem._par.coefficient
import mfem._par.globals
import mfem._par.matrix
import mfem._par.intrules
import mfem._par.sparsemat
import mfem._par.densemat
import mfem._par.eltrans
import mfem._par.fe
import mfem._par.geom
import mfem._par.mesh
import mfem._par.sort_pairs
import mfem._par.ncmesh
import mfem._par.vtk
import mfem._par.element
import mfem._par.table
import mfem._par.hash
import mfem._par.vertex
import mfem._par.gridfunc
import mfem._par.bilininteg
import mfem._par.fe_coll
import mfem._par.lininteg
import mfem._par.linearform
import mfem._par.handle
import mfem._par.hypre
import mfem._par.restriction
import mfem._par.pmesh
import mfem._par.pncmesh
import mfem._par.communication
import mfem._par.sets
class ParFiniteElementSpace(mfem._par.fespace.FiniteElementSpace):
    r"""Proxy of C++ mfem::ParFiniteElementSpace class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr
    num_face_nbr_dofs = property(_pfespace.ParFiniteElementSpace_num_face_nbr_dofs_get, _pfespace.ParFiniteElementSpace_num_face_nbr_dofs_set, doc=r"""num_face_nbr_dofs : int""")
    face_nbr_element_dof = property(_pfespace.ParFiniteElementSpace_face_nbr_element_dof_get, _pfespace.ParFiniteElementSpace_face_nbr_element_dof_set, doc=r"""face_nbr_element_dof : mfem::Table""")
    face_nbr_ldof = property(_pfespace.ParFiniteElementSpace_face_nbr_ldof_get, _pfespace.ParFiniteElementSpace_face_nbr_ldof_set, doc=r"""face_nbr_ldof : mfem::Table""")
    face_nbr_glob_dof_map = property(_pfespace.ParFiniteElementSpace_face_nbr_glob_dof_map_get, doc=r"""face_nbr_glob_dof_map : mfem::Array<(HYPRE_Int)>""")
    send_face_nbr_ldof = property(_pfespace.ParFiniteElementSpace_send_face_nbr_ldof_get, _pfespace.ParFiniteElementSpace_send_face_nbr_ldof_set, doc=r"""send_face_nbr_ldof : mfem::Table""")

    def __init__(self, *args):
        r"""
        __init__(ParFiniteElementSpace self, ParFiniteElementSpace orig, ParMesh pmesh=None, FiniteElementCollection fec=None) -> ParFiniteElementSpace
        __init__(ParFiniteElementSpace self, FiniteElementSpace orig, ParMesh pmesh, FiniteElementCollection fec=None) -> ParFiniteElementSpace
        __init__(ParFiniteElementSpace self, ParMesh pm, FiniteElementSpace global_fes, int const * partitioning, FiniteElementCollection f=None) -> ParFiniteElementSpace
        __init__(ParFiniteElementSpace self, ParMesh pm, FiniteElementCollection f, int dim=1, int ordering=byNODES) -> ParFiniteElementSpace
        __init__(ParFiniteElementSpace self, ParMesh pm, mfem::NURBSExtension * ext, FiniteElementCollection f, int dim=1, int ordering=byNODES) -> ParFiniteElementSpace
        """
        _pfespace.ParFiniteElementSpace_swiginit(self, _pfespace.new_ParFiniteElementSpace(*args))

    def GetComm(self):
        r"""GetComm(ParFiniteElementSpace self) -> MPI_Comm"""
        return _pfespace.ParFiniteElementSpace_GetComm(self)
    GetComm = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetComm)

    def GetNRanks(self):
        r"""GetNRanks(ParFiniteElementSpace self) -> int"""
        return _pfespace.ParFiniteElementSpace_GetNRanks(self)
    GetNRanks = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetNRanks)

    def GetMyRank(self):
        r"""GetMyRank(ParFiniteElementSpace self) -> int"""
        return _pfespace.ParFiniteElementSpace_GetMyRank(self)
    GetMyRank = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetMyRank)

    def GetParMesh(self):
        r"""GetParMesh(ParFiniteElementSpace self) -> ParMesh"""
        return _pfespace.ParFiniteElementSpace_GetParMesh(self)
    GetParMesh = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetParMesh)

    def GetDofSign(self, i):
        r"""GetDofSign(ParFiniteElementSpace self, int i) -> int"""
        return _pfespace.ParFiniteElementSpace_GetDofSign(self, i)
    GetDofSign = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetDofSign)

    def GetDofOffsets(self):
        r"""GetDofOffsets(ParFiniteElementSpace self) -> HYPRE_Int *"""
        return _pfespace.ParFiniteElementSpace_GetDofOffsets(self)
    GetDofOffsets = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetDofOffsets)

    def GetTrueDofOffsets(self):
        r"""GetTrueDofOffsets(ParFiniteElementSpace self) -> HYPRE_Int *"""
        return _pfespace.ParFiniteElementSpace_GetTrueDofOffsets(self)
    GetTrueDofOffsets = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetTrueDofOffsets)

    def GlobalVSize(self):
        r"""GlobalVSize(ParFiniteElementSpace self) -> HYPRE_Int"""
        return _pfespace.ParFiniteElementSpace_GlobalVSize(self)
    GlobalVSize = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GlobalVSize)

    def GlobalTrueVSize(self):
        r"""GlobalTrueVSize(ParFiniteElementSpace self) -> HYPRE_Int"""
        return _pfespace.ParFiniteElementSpace_GlobalTrueVSize(self)
    GlobalTrueVSize = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GlobalTrueVSize)

    def GetTrueVSize(self):
        r"""GetTrueVSize(ParFiniteElementSpace self) -> int"""
        return _pfespace.ParFiniteElementSpace_GetTrueVSize(self)
    GetTrueVSize = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetTrueVSize)

    def GetElementDofs(self, i):
        from  .array import intArray
        vdofs = intArray()
        _pfespace.ParFiniteElementSpace_GetElementDofs(self, i, vdofs)
        return vdofs.ToList()



    def GetBdrElementDofs(self, i):
        from  .array import intArray
        vdofs = intArray()
        _pfespace.ParFiniteElementSpace_GetBdrElementDofs(self, i, vdofs)
        return vdofs.ToList()



    def GetFaceDofs(self, i):
        from  .array import intArray
        vdofs = intArray()
        _pfespace.ParFiniteElementSpace_GetFaceDofs(self, i, vdofs)
        return vdofs.ToList()



    def GetFaceRestriction(self, *args, **kwargs):
        r"""GetFaceRestriction(ParFiniteElementSpace self, mfem::ElementDofOrdering e_ordering, mfem::FaceType type, mfem::L2FaceValues mul=DoubleValued) -> Operator"""
        return _pfespace.ParFiniteElementSpace_GetFaceRestriction(self, *args, **kwargs)
    GetFaceRestriction = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetFaceRestriction)

    def GetSharedEdgeDofs(self, group, ei):
      from  .array import intArray
      dofs = intArray()      
      _pfespace.ParFiniteElementSpace_GetSharedEdgeDofs(self, group, ei, dofs)
      return dofs.ToList()      



    def GetSharedTriangleDofs(self, group, fi, dofs):
        r"""GetSharedTriangleDofs(ParFiniteElementSpace self, int group, int fi, intArray dofs)"""
        return _pfespace.ParFiniteElementSpace_GetSharedTriangleDofs(self, group, fi, dofs)
    GetSharedTriangleDofs = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetSharedTriangleDofs)

    def GetSharedQuadrilateralDofs(self, group, fi, dofs):
        r"""GetSharedQuadrilateralDofs(ParFiniteElementSpace self, int group, int fi, intArray dofs)"""
        return _pfespace.ParFiniteElementSpace_GetSharedQuadrilateralDofs(self, group, fi, dofs)
    GetSharedQuadrilateralDofs = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetSharedQuadrilateralDofs)

    def Dof_TrueDof_Matrix(self):
        r"""Dof_TrueDof_Matrix(ParFiniteElementSpace self) -> HypreParMatrix"""
        return _pfespace.ParFiniteElementSpace_Dof_TrueDof_Matrix(self)
    Dof_TrueDof_Matrix = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_Dof_TrueDof_Matrix)

    def GetPartialConformingInterpolation(self):
        r"""GetPartialConformingInterpolation(ParFiniteElementSpace self) -> HypreParMatrix"""
        return _pfespace.ParFiniteElementSpace_GetPartialConformingInterpolation(self)
    GetPartialConformingInterpolation = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetPartialConformingInterpolation)

    def NewTrueDofVector(self):
        r"""NewTrueDofVector(ParFiniteElementSpace self) -> HypreParVector"""
        return _pfespace.ParFiniteElementSpace_NewTrueDofVector(self)
    NewTrueDofVector = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_NewTrueDofVector)

    def DivideByGroupSize(self, vec):
        r"""DivideByGroupSize(ParFiniteElementSpace self, double * vec)"""
        return _pfespace.ParFiniteElementSpace_DivideByGroupSize(self, vec)
    DivideByGroupSize = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_DivideByGroupSize)

    def GroupComm(self, *args):
        r"""
        GroupComm(ParFiniteElementSpace self) -> GroupCommunicator
        GroupComm(ParFiniteElementSpace self) -> GroupCommunicator
        """
        return _pfespace.ParFiniteElementSpace_GroupComm(self, *args)
    GroupComm = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GroupComm)

    def ScalarGroupComm(self):
        r"""ScalarGroupComm(ParFiniteElementSpace self) -> GroupCommunicator"""
        return _pfespace.ParFiniteElementSpace_ScalarGroupComm(self)
    ScalarGroupComm = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_ScalarGroupComm)

    def Synchronize(self, ldof_marker):
        r"""Synchronize(ParFiniteElementSpace self, intArray ldof_marker)"""
        return _pfespace.ParFiniteElementSpace_Synchronize(self, ldof_marker)
    Synchronize = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_Synchronize)

    def GetEssentialVDofs(self, bdr_attr_is_ess, ess_dofs, component=-1):
        r"""GetEssentialVDofs(ParFiniteElementSpace self, intArray bdr_attr_is_ess, intArray ess_dofs, int component=-1)"""
        return _pfespace.ParFiniteElementSpace_GetEssentialVDofs(self, bdr_attr_is_ess, ess_dofs, component)
    GetEssentialVDofs = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetEssentialVDofs)

    def GetEssentialTrueDofs(self, bdr_attr_is_ess, ess_tdof_list, component=-1):
        r"""GetEssentialTrueDofs(ParFiniteElementSpace self, intArray bdr_attr_is_ess, intArray ess_tdof_list, int component=-1)"""
        return _pfespace.ParFiniteElementSpace_GetEssentialTrueDofs(self, bdr_attr_is_ess, ess_tdof_list, component)
    GetEssentialTrueDofs = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetEssentialTrueDofs)

    def GetLocalTDofNumber(self, ldof):
        r"""GetLocalTDofNumber(ParFiniteElementSpace self, int ldof) -> int"""
        return _pfespace.ParFiniteElementSpace_GetLocalTDofNumber(self, ldof)
    GetLocalTDofNumber = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetLocalTDofNumber)

    def GetGlobalTDofNumber(self, ldof):
        r"""GetGlobalTDofNumber(ParFiniteElementSpace self, int ldof) -> HYPRE_Int"""
        return _pfespace.ParFiniteElementSpace_GetGlobalTDofNumber(self, ldof)
    GetGlobalTDofNumber = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetGlobalTDofNumber)

    def GetGlobalScalarTDofNumber(self, sldof):
        r"""GetGlobalScalarTDofNumber(ParFiniteElementSpace self, int sldof) -> HYPRE_Int"""
        return _pfespace.ParFiniteElementSpace_GetGlobalScalarTDofNumber(self, sldof)
    GetGlobalScalarTDofNumber = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetGlobalScalarTDofNumber)

    def GetMyDofOffset(self):
        r"""GetMyDofOffset(ParFiniteElementSpace self) -> HYPRE_Int"""
        return _pfespace.ParFiniteElementSpace_GetMyDofOffset(self)
    GetMyDofOffset = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetMyDofOffset)

    def GetMyTDofOffset(self):
        r"""GetMyTDofOffset(ParFiniteElementSpace self) -> HYPRE_Int"""
        return _pfespace.ParFiniteElementSpace_GetMyTDofOffset(self)
    GetMyTDofOffset = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetMyTDofOffset)

    def GetProlongationMatrix(self):
        r"""GetProlongationMatrix(ParFiniteElementSpace self) -> Operator"""
        return _pfespace.ParFiniteElementSpace_GetProlongationMatrix(self)
    GetProlongationMatrix = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetProlongationMatrix)

    def GetRestrictionMatrix(self):
        r"""GetRestrictionMatrix(ParFiniteElementSpace self) -> SparseMatrix"""
        return _pfespace.ParFiniteElementSpace_GetRestrictionMatrix(self)
    GetRestrictionMatrix = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetRestrictionMatrix)

    def ExchangeFaceNbrData(self):
        r"""ExchangeFaceNbrData(ParFiniteElementSpace self)"""
        return _pfespace.ParFiniteElementSpace_ExchangeFaceNbrData(self)
    ExchangeFaceNbrData = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_ExchangeFaceNbrData)

    def GetFaceNbrVSize(self):
        r"""GetFaceNbrVSize(ParFiniteElementSpace self) -> int"""
        return _pfespace.ParFiniteElementSpace_GetFaceNbrVSize(self)
    GetFaceNbrVSize = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetFaceNbrVSize)

    def GetFaceNbrElementVDofs(self, i, vdofs):
        r"""GetFaceNbrElementVDofs(ParFiniteElementSpace self, int i, intArray vdofs)"""
        return _pfespace.ParFiniteElementSpace_GetFaceNbrElementVDofs(self, i, vdofs)
    GetFaceNbrElementVDofs = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetFaceNbrElementVDofs)

    def GetFaceNbrFaceVDofs(self, i, vdofs):
        r"""GetFaceNbrFaceVDofs(ParFiniteElementSpace self, int i, intArray vdofs)"""
        return _pfespace.ParFiniteElementSpace_GetFaceNbrFaceVDofs(self, i, vdofs)
    GetFaceNbrFaceVDofs = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetFaceNbrFaceVDofs)

    def GetFaceNbrFE(self, i):
        r"""GetFaceNbrFE(ParFiniteElementSpace self, int i) -> FiniteElement"""
        return _pfespace.ParFiniteElementSpace_GetFaceNbrFE(self, i)
    GetFaceNbrFE = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetFaceNbrFE)

    def GetFaceNbrFaceFE(self, i):
        r"""GetFaceNbrFaceFE(ParFiniteElementSpace self, int i) -> FiniteElement"""
        return _pfespace.ParFiniteElementSpace_GetFaceNbrFaceFE(self, i)
    GetFaceNbrFaceFE = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetFaceNbrFaceFE)

    def GetFaceNbrGlobalDofMap(self):
        r"""GetFaceNbrGlobalDofMap(ParFiniteElementSpace self) -> HYPRE_Int const *"""
        return _pfespace.ParFiniteElementSpace_GetFaceNbrGlobalDofMap(self)
    GetFaceNbrGlobalDofMap = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetFaceNbrGlobalDofMap)

    def GetFaceNbrElementTransformation(self, i):
        r"""GetFaceNbrElementTransformation(ParFiniteElementSpace self, int i) -> ElementTransformation"""
        return _pfespace.ParFiniteElementSpace_GetFaceNbrElementTransformation(self, i)
    GetFaceNbrElementTransformation = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetFaceNbrElementTransformation)

    def Lose_Dof_TrueDof_Matrix(self):
        r"""Lose_Dof_TrueDof_Matrix(ParFiniteElementSpace self)"""
        return _pfespace.ParFiniteElementSpace_Lose_Dof_TrueDof_Matrix(self)
    Lose_Dof_TrueDof_Matrix = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_Lose_Dof_TrueDof_Matrix)

    def LoseDofOffsets(self):
        r"""LoseDofOffsets(ParFiniteElementSpace self)"""
        return _pfespace.ParFiniteElementSpace_LoseDofOffsets(self)
    LoseDofOffsets = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_LoseDofOffsets)

    def LoseTrueDofOffsets(self):
        r"""LoseTrueDofOffsets(ParFiniteElementSpace self)"""
        return _pfespace.ParFiniteElementSpace_LoseTrueDofOffsets(self)
    LoseTrueDofOffsets = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_LoseTrueDofOffsets)

    def Conforming(self):
        r"""Conforming(ParFiniteElementSpace self) -> bool"""
        return _pfespace.ParFiniteElementSpace_Conforming(self)
    Conforming = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_Conforming)

    def Nonconforming(self):
        r"""Nonconforming(ParFiniteElementSpace self) -> bool"""
        return _pfespace.ParFiniteElementSpace_Nonconforming(self)
    Nonconforming = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_Nonconforming)

    def GetTrueTransferOperator(self, coarse_fes, T):
        r"""GetTrueTransferOperator(ParFiniteElementSpace self, FiniteElementSpace coarse_fes, OperatorHandle T)"""
        return _pfespace.ParFiniteElementSpace_GetTrueTransferOperator(self, coarse_fes, T)
    GetTrueTransferOperator = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_GetTrueTransferOperator)

    def Update(self, want_transform=True):
        r"""Update(ParFiniteElementSpace self, bool want_transform=True)"""
        return _pfespace.ParFiniteElementSpace_Update(self, want_transform)
    Update = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_Update)

    def UpdatesFinished(self):
        r"""UpdatesFinished(ParFiniteElementSpace self)"""
        return _pfespace.ParFiniteElementSpace_UpdatesFinished(self)
    UpdatesFinished = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_UpdatesFinished)
    __swig_destroy__ = _pfespace.delete_ParFiniteElementSpace

    def PrintPartitionStats(self):
        r"""PrintPartitionStats(ParFiniteElementSpace self)"""
        return _pfespace.ParFiniteElementSpace_PrintPartitionStats(self)
    PrintPartitionStats = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_PrintPartitionStats)

    def TrueVSize(self):
        r"""TrueVSize(ParFiniteElementSpace self) -> int"""
        return _pfespace.ParFiniteElementSpace_TrueVSize(self)
    TrueVSize = _swig_new_instance_method(_pfespace.ParFiniteElementSpace_TrueVSize)

# Register ParFiniteElementSpace in _pfespace:
_pfespace.ParFiniteElementSpace_swigregister(ParFiniteElementSpace)

class ConformingProlongationOperator(mfem._par.operators.Operator):
    r"""Proxy of C++ mfem::ConformingProlongationOperator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, pfes):
        r"""__init__(ConformingProlongationOperator self, ParFiniteElementSpace pfes) -> ConformingProlongationOperator"""
        _pfespace.ConformingProlongationOperator_swiginit(self, _pfespace.new_ConformingProlongationOperator(pfes))

    def Mult(self, x, y):
        r"""Mult(ConformingProlongationOperator self, Vector x, Vector y)"""
        return _pfespace.ConformingProlongationOperator_Mult(self, x, y)
    Mult = _swig_new_instance_method(_pfespace.ConformingProlongationOperator_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(ConformingProlongationOperator self, Vector x, Vector y)"""
        return _pfespace.ConformingProlongationOperator_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_pfespace.ConformingProlongationOperator_MultTranspose)
    __swig_destroy__ = _pfespace.delete_ConformingProlongationOperator

# Register ConformingProlongationOperator in _pfespace:
_pfespace.ConformingProlongationOperator_swigregister(ConformingProlongationOperator)

class DeviceConformingProlongationOperator(ConformingProlongationOperator):
    r"""Proxy of C++ mfem::DeviceConformingProlongationOperator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, pfes):
        r"""__init__(DeviceConformingProlongationOperator self, ParFiniteElementSpace pfes) -> DeviceConformingProlongationOperator"""
        _pfespace.DeviceConformingProlongationOperator_swiginit(self, _pfespace.new_DeviceConformingProlongationOperator(pfes))
    __swig_destroy__ = _pfespace.delete_DeviceConformingProlongationOperator

    def Mult(self, x, y):
        r"""Mult(DeviceConformingProlongationOperator self, Vector x, Vector y)"""
        return _pfespace.DeviceConformingProlongationOperator_Mult(self, x, y)
    Mult = _swig_new_instance_method(_pfespace.DeviceConformingProlongationOperator_Mult)

    def MultTranspose(self, x, y):
        r"""MultTranspose(DeviceConformingProlongationOperator self, Vector x, Vector y)"""
        return _pfespace.DeviceConformingProlongationOperator_MultTranspose(self, x, y)
    MultTranspose = _swig_new_instance_method(_pfespace.DeviceConformingProlongationOperator_MultTranspose)

# Register DeviceConformingProlongationOperator in _pfespace:
_pfespace.DeviceConformingProlongationOperator_swigregister(DeviceConformingProlongationOperator)



